package ml2java.parser;

import java.io.Reader;
import java.io.FileReader;
import java.io.IOException;
import java.util.NoSuchElementException;
import java.util.Objects;

import ml2java.util.*;

import static ml2java.util.SimpleList.*;
import static ml2java.util.Pair.*;
import static ml2java.util.Option.*;
import static ml2java.util.Bind.*;

import static ml2java.parser.Operation.*;

public abstract class Exp {
    
    public static Exp IntCst(Integer field1) {
        return new IntCst(field1);
    }
    public static Exp Var(String field1) {
        return new Var(field1);
    }
    public static Exp Oper(Operation field1, Exp field2, Exp field3) {
        return new Oper(field1, field2, field3);
    }
    
    public abstract <R> R match(Function1<Integer, R> f1, Function1<String, R> f2, Function3<Operation, Exp, Exp, R> f3);
    
    public abstract <R> R accept(ExpVisitor<R> v);
    
    public interface ExpVisitor<R> {
    
        public R visitIntCst(Integer i);
        public R visitVar(String s);
        public R visitOper(Operation o, Exp e1, Exp e2);
    
    }
    public static abstract class ExpDefaultVisitor<R> implements ExpVisitor<R> {
    
        public R visitIntCst(Integer i) {
            return visitDefault();
        }
        public R visitVar(String s) {
            return visitDefault();
        }
        public R visitOper(Operation o, Exp e1, Exp e2) {
            return visitDefault();
        }
        public abstract R visitDefault();
    
    }
}

class IntCst extends Exp { 
    final Integer field1;
    public IntCst(Integer field1) {
        this.field1 = field1;
    }
    @Override
    public int hashCode() {
        return Objects.hash(field1);
    }
    @Override
    public boolean equals(Object other) {
        if (other == null) return false;
        return (this.getClass() == (other.getClass())
                    && this.field1.equals(((IntCst)other).field1));
    }
    public <R> R match(Function1<Integer, R> f1, Function1<String, R> f2, Function3<Operation, Exp, Exp, R> f3) {
        return f1.apply(field1);
    }
    public <R> R accept(ExpVisitor<R> v) {
        return v.visitIntCst(field1);
    }
}
class Var extends Exp { 
    final String field1;
    public Var(String field1) {
        this.field1 = field1;
    }
    @Override
    public int hashCode() {
        return Objects.hash(field1);
    }
    @Override
    public boolean equals(Object other) {
        if (other == null) return false;
        return (this.getClass() == (other.getClass())
                    && this.field1.equals(((Var)other).field1));
    }
    public <R> R match(Function1<Integer, R> f1, Function1<String, R> f2, Function3<Operation, Exp, Exp, R> f3) {
        return f2.apply(field1);
    }
    public <R> R accept(ExpVisitor<R> v) {
        return v.visitVar(field1);
    }
}
class Oper extends Exp { 
    final Operation field1;
    final Exp field2;
    final Exp field3;
    public Oper(Operation field1, Exp field2, Exp field3) {
        this.field1 = field1;
        this.field2 = field2;
        this.field3 = field3;
    }
    @Override
    public int hashCode() {
        return Objects.hash(field1, field2, field3);
    }
    @Override
    public boolean equals(Object other) {
        if (other == null) return false;
        return (this.getClass() == (other.getClass())
                    && this.field1.equals(((Oper)other).field1)
                    && this.field2.equals(((Oper)other).field2)
                    && this.field3.equals(((Oper)other).field3));
    }
    public <R> R match(Function1<Integer, R> f1, Function1<String, R> f2, Function3<Operation, Exp, Exp, R> f3) {
        return f3.apply(field1, field2, field3);
    }
    public <R> R accept(ExpVisitor<R> v) {
        return v.visitOper(field1, field2, field3);
    }
}

